"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const minimist = require("minimist");
const path = require("path");
const cli_utils_1 = require("@ionic/cli-utils");
const plugins_1 = require("./lib/plugins");
const PROJECT_FILE = 'ionic.config.json';
function cleanup() {
    for (let task of cli_utils_1.TASKS) {
        if (task.running) {
            task.fail();
        }
        task.clear();
    }
}
function run(pargv, env) {
    return __awaiter(this, void 0, void 0, function* () {
        let exitCode = 0;
        let err;
        // Check version?
        pargv = pargv.slice(2);
        const argv = minimist(pargv);
        const log = new cli_utils_1.Logger();
        // If verbose flag is passed the log.level becomes debug
        if (argv['verbose']) {
            argv['log-level'] = 'debug';
        }
        if (argv['log-level']) {
            log.level = argv['log-level'];
        }
        env['PROJECT_FILE'] = PROJECT_FILE;
        env['PROJECT_DIR'] = yield getProjectRootDir(process.cwd(), env['PROJECT_FILE']);
        try {
            const [plugin, inputs] = yield plugins_1.resolvePlugin(env['PROJECT_DIR'], pargv);
            yield plugin.run(inputs, env);
        }
        catch (e) {
            err = e;
        }
        cleanup();
        if (err) {
            exitCode = 1;
            if (cli_utils_1.isSuperAgentError(err)) {
                console.error(cli_utils_1.formatError(err));
            }
            else if (err instanceof cli_utils_1.FatalException) {
                exitCode = err.exitCode;
                if (err.message) {
                    log.error(err.message);
                }
            }
            else {
                console.error(err);
            }
            process.exit(exitCode);
        }
    });
}
exports.run = run;
/**
 * Find the base project directory based on the dir input
 */
function getProjectRootDir(dir, projectFileName) {
    return __awaiter(this, void 0, void 0, function* () {
        const dirInfo = path.parse(dir);
        const directoriesToCheck = dirInfo.dir
            .slice(dirInfo.root.length)
            .split(path.sep)
            .concat(dirInfo.base)
            .map((segment, index, array) => {
            let pathSegments = array.slice(0, (array.length - index));
            return dirInfo.root + path.join(...pathSegments);
        });
        for (let i = 0; i < directoriesToCheck.length; i++) {
            const results = yield cli_utils_1.fsReadDir(directoriesToCheck[i]);
            if (results.includes(projectFileName)) {
                return directoriesToCheck[i];
            }
        }
        return '';
    });
}
